import 'package:bokain_models/src/model_base.dart';

class Day extends ModelBase
{
  static const int startHour = 8;
  static const int startMinute = 0;
  static const int endHour = 20;
  static const int endMinute = 0;

  Day(this.userId, DateTime start_date) : super()
  {
    DateTime startTime = new DateTime(
        start_date.year,
        start_date.month,
        start_date.day,
        startHour,
        startMinute);

    DateTime endTime = new DateTime(
        start_date.year,
        start_date.month,
        start_date.day,
        endHour,
        endMinute);

    DateTime iTime = startTime;
    while (iTime.isBefore(endTime))
    {
      increments.add(new Increment(iTime));
      iTime = iTime.add(Increment.duration);
    }
  }

  Day.parse(Map<String, dynamic> data) : super.parse(data);

  @override
  void parseData(Map<String, dynamic> data)
  {
    increments.clear();
    userId = data["userId"];

    // Fetch active increments
    Iterable<Map<String, String>> activeIncrementsData = data["activeIncrements"];
    List<Increment> activeIncrements = new List();
    if (activeIncrementsData != null)
    {
      for (Map<String, dynamic> incrementData in activeIncrementsData)
      {
        activeIncrements.add(new Increment.parse(incrementData));
      }
    }

    // Create all increments and merge in active ones
    DateTime iTime = DateTime.parse(data["startTime"]);
    DateTime endTime = new DateTime(iTime.year, iTime.month, iTime.day, endHour, endMinute);

    while (iTime.isBefore(endTime))
    {
      Increment inc = activeIncrements.firstWhere((i) => i.startTime.isAtSameMomentAs(iTime), orElse: () => null);
      if (inc == null) inc = new Increment(iTime);
      increments.add(inc);
      iTime = iTime.add(Increment.duration);
    }
  }

  @override
  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["userId"] = userId;
    data["startTime"] = startTime.toIso8601String();
    Iterable<Increment> activeIncrements = increments.where((i) => i.state != null);
    data["activeIncrements"] = activeIncrements.map((i) => i.properties).toList(growable: false);
    return data;
  }

  bool isSameDateAs(DateTime dt)
  {
    DateTime start = increments.first.startTime;
    return (start.year == dt.year && start.month == dt.month && start.day == dt.day);
  }

  DateTime get startTime => increments.first.startTime;
  DateTime get endTime => increments.last.endTime;

  String userId;
  List<Increment> increments = new List();
}

class Increment
{
  static const Duration duration = const Duration(minutes: 10);

  Increment(this._startTime)
  {
    _endTime = _startTime.add(duration);
  }

  Increment.parse(Map<String, dynamic> data)
  {
    _startTime = DateTime.parse(data["startTime"]);
    _endTime = _startTime.add(duration);
    state = data["state"];
    bookingId = data["bookingId"];
    roomId = data["roomId"];
  }

  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["startTime"] = _startTime.toIso8601String();
    data["endTime"] = _endTime.toIso8601String();
    data["bookingId"] = bookingId;
    data["roomId"] = roomId;
    data["state"] = state;
    return data;
  }

  DateTime get startTime => _startTime;
  DateTime get endTime => _endTime;

  DateTime _startTime;
  DateTime _endTime;

  String state;
  String bookingId;
  String roomId;

  bool highlighted = false;
}